// Ball Code - As of 5/1/15
// Written by Alice Kassar and Jesse Checkla
// Portions of this code are borrowed from ManiacBug and Adafruit
// See the references for more information.

// Include libraries:
#include <Wire.h>
#include <Adafruit_ADXL345_U.h>
#include <Adafruit_Sensor.h>
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>
#include <Time.h>

// Declare constants:
#define CE_PIN   9
#define CSN_PIN 10

// NOTE: the "LL" at the end of the constant is "LongLong" type
//const uint64_t pipe = 0xE8E8F0F0E1LL; // Define the transmit pipe
byte addresses[][6] = {"1Node", "2Node"};

// Create a Radio Object
RF24 radio(CE_PIN, CSN_PIN);
Adafruit_ADXL345_Unified ADXL = Adafruit_ADXL345_Unified(12345);

// Declare Variables:
int hold;
int holdnbr = 0;
int ballid = 3;
double reference;

byte int_source;
char tapFF = 0;
//LED Pins
int LED1 = 4;
int LED2 = 5;
int LED3 = 6;
//int CS_ADXL=6;
unsigned long int start = 0;

// The event structure:
struct dataStruct {
  unsigned long _micros;
  unsigned long time;
  int id = 0;
  float interrupt;
} myData;

struct dStruct {
  int LEDnbr;
} LEDData;

//ADXL345 Register Addresses
#define	DEVID		0x00	//Device ID Register
#define THRESH_TAP	0x1D	//Tap Threshold
#define	OFSX		0x1E	//X-axis offset
#define	OFSY		0x1F	//Y-axis offset
#define	OFSZ		0x20	//Z-axis offset
#define	DURATION	0x21	//Tap Duration
#define	LATENT		0x22	//Tap latency
#define	WINDOW		0x23	//Tap window
#define	THRESH_ACT	0x24	//Activity Threshold
#define	THRESH_INACT	0x25	//Inactivity Threshold
#define	TIME_INACT	0x26	//Inactivity Time
#define	ACT_INACT_CTL	0x27	//Axis enable control for activity and inactivity detection
#define	THRESH_FF	0x28	//free-fall threshold
#define	TIME_FF		0x29	//Free-Fall Time
#define	TAP_AXES	0x2A	//Axis control for tap/double tap
#define ACT_TAP_STATUS	0x2B	//Source of tap/double tap
#define	BW_RATE		0x2C	//Data rate and power mode control
#define POWER_CTL	0x2D	//Power Control Register
#define	INT_ENABLE	0x2E	//Interrupt Enable Control
#define	INT_MAP		0x2F	//Interrupt Mapping Control
#define	INT_SOURCE	0x30	//Source of interrupts
#define	DATA_FORMAT	0x31	//Data format control
#define DATAX0		0x32	//X-Axis Data 0
#define DATAX1		0x33	//X-Axis Data 1
#define DATAY0		0x34	//Y-Axis Data 0
#define DATAY1		0x35	//Y-Axis Data 1
#define DATAZ0		0x36	//Z-Axis Data 0
#define DATAZ1		0x37	//Z-Axis Data 1
#define	FIFO_CTL	0x38	//FIFO control
#define	FIFO_STATUS	0x39	//FIFO status



void setup()
{
  // Initialize the I2C
  Wire.begin();

  // Open Serial Communication:
  Serial.begin(9600);

  // Initialize the accelerometer registers/thresholds
  ADXL.writeRegister(ADXL345_REG_THRESH_TAP, 0b11001000);
  ADXL.writeRegister(ADXL345_REG_DUR, 0x08);
  ADXL.writeRegister(ADXL345_REG_LATENT, 0x00);
  ADXL.writeRegister(ADXL345_REG_WINDOW, 0x00);
  ADXL.writeRegister(ADXL345_REG_TAP_AXES, 0b00000111);
  ADXL.writeRegister(ADXL345_REG_POWER_CTL, 0x08);
  ADXL.writeRegister(ADXL345_REG_INT_MAP, 0b10111011);
  ADXL.writeRegister(ADXL345_REG_DATA_FORMAT, 0x01);
  ADXL.writeRegister(ADXL345_REG_INT_ENABLE, 0b11000100);
  ADXL.writeRegister(ADXL345_REG_THRESH_FF, 0x05);
  ADXL.writeRegister(ADXL345_REG_TIME_FF, 0x05);

  //Set LED Pins as Output
  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(LED3, OUTPUT);

  //  pinMode(CS_ADXL, OUTPUT);
  //digitalWrite(CS_ADXL, HIGH);

  // Set LEDs LOW
  digitalWrite(LED1, LOW);
  digitalWrite(LED2, LOW);
  digitalWrite(LED3, LOW);

  delay(10);

  //Initialize the interrupts
  int_source = ADXL.readRegister(ADXL345_REG_INT_SOURCE);
  attachInterrupt(1, TapFFinterrupt, RISING);

  //Start Radio Communication
  radio.begin();

  // Set the PA Level low to prevent power supply related issues since this is a
  // getting_started sketch, and the likelihood of close proximity of the devices. RF24_PA_MAX is default.
  radio.setPALevel(RF24_PA_LOW);
  radio.openWritingPipe(addresses[1]);
  radio.openReadingPipe(1, addresses[0]);

  radio.startListening();

  // Wait for starting seed:
  while (start == 0)
  {
    radio.read( &myData, sizeof(myData));
    start = myData._micros;
  }
  myData.id = ballid;
  radio.stopListening();

  // Event timer (no longer in use, JC).
  reference = millis();
}

void loop()
{

  if (int_source == 255)
  {
    int_source = ADXL.readRegister( ADXL345_REG_INT_SOURCE );
  }

  if (tapFF == 1)
  {

    int_source = ADXL.readRegister( ADXL345_REG_INT_SOURCE );
    tapFF = 0;
    if (int_source == 195)
    {
      myData.interrupt = 1;
      ADXL.writeRegister(ADXL345_REG_INT_ENABLE, 0b11000100);
    }
    else if (int_source == 135)
    {
      myData.interrupt = 2;
      ADXL.writeRegister(ADXL345_REG_INT_ENABLE, 0b11000000);
    }
    else myData.interrupt = 3;

    myData._micros = micros();
    if (int_source != holdnbr && myData.interrupt != 3)
    {
      radio.write(&myData, sizeof(myData));
    }
    if (myData.interrupt != 3)
      holdnbr = int_source;

  }

  radio.startListening();
  unsigned long started_waiting_at = micros();
  boolean timeout = false;
  // While nothing is received
  while ( ! radio.available() )
  {
    // Timeout after 500ms, exit while loop
    if (micros() - started_waiting_at > 10 )
    {
      timeout = true;
      break;
    }
  }

  if (timeout)
  {
    Serial.println(F("No msg from receiver."));
  }

  else
  {
    radio.read( &LEDData, sizeof(LEDData));
    if (LEDData.LEDnbr == 10)
    {
      digitalWrite(LED1, LOW);
      digitalWrite(LED2, LOW);
      // digitalWrite(LED3, LOW);
    }
    else if (LEDData.LEDnbr == 1)
    {
      digitalWrite(LED1, HIGH);
      digitalWrite(LED2, LOW);
      digitalWrite(LED3, LOW);
    }
    else if (LEDData.LEDnbr == 2)
    {
      digitalWrite(LED1, LOW);
      digitalWrite(LED2, HIGH);
      digitalWrite(LED3, LOW);
    }
    else if (LEDData.LEDnbr == 3)
    {
      digitalWrite(LED1, LOW);
      digitalWrite(LED2, LOW);
      digitalWrite(LED3, HIGH);
    }
  }
  radio.stopListening();
} // End Loop

// ISR
void TapFFinterrupt()
{
  myData.time = millis() - reference;
  tapFF = 1;
}


