// Receiver Code - As of 5/1/15
// Written by Jesse Checkla and Alice Kassar
// Portions of this code are borrowed from ManiacBug and Adafruit

// Import Libraries:
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>
#include <SD.h>
#include <QueueArray.h>

// Declare Variables:
#define CE_PIN   9
#define CSN_PIN 10

struct dataStruct {
  volatile unsigned long _micros = 1;
  volatile  unsigned long time;
  volatile  int id = 0;
  volatile  float interrupt;
};

dataStruct myData;
dataStruct rData ;
QueueArray <dataStruct> queue;

struct dStruct {
  int LEDnbr;
} LEDData;

//File myFile;
float reference;
float timeellapsed;
int counter = 0;
char pattern;

// NOTE: the "LL" at the end of the constant is "LongLong" type
//const uint64_t pipe = 0xE8E8F0F0E1LL; // Define the transmit pipe
byte addresses[][6] = {"1Node", "2Node"};

// Create a Radio Object
RF24 radio(CE_PIN, CSN_PIN); 

//timer 0 compare ISR
ISR (TIMER0_COMPA_vect)
{
  //Serial.println("interrput ");
  if (radio.available() )
  {
    radio.read( &rData, sizeof(rData)  );
    if (rData.interrupt == 2 || rData.interrupt == 1)
    {
      queue.push (rData);
      // Serial.println(rData.interrupt);

    }
    //Decrement the three times if they are not already zero

  }

}

void setup()  
{
  
  // Start Serial:
  Serial.begin(9600);
  delay(1000);
  
  // Start Radio Communication
  radio.begin();
  // Serial.println("Nrf24L01 Receiver Starting");
  
  // Set the PA Level low to prevent power supply related issues since this is a
  // getting_started sketch, and the likelihood of close proximity of the devices. RF24_PA_MAX is default.
  radio.setPALevel(RF24_PA_LOW);
  radio.openWritingPipe(addresses[0]);
  radio.openReadingPipe(1, addresses[1]);

  // Send Starting Seed
  radio.write(&myData, sizeof(myData) );
  LEDData.LEDnbr = 10;
  delay(1);
  radio.write(&LEDData, sizeof(LEDData) );
  Serial.println("sending seed");
  radio.startListening();
  reference = millis();
  delay(100);
  
  //turn on timer 0 cmp match ISR
  TIMSK0 = (1 << OCIE0A);	
  
  //set the compare reg to 20 time ticks
  OCR0A = 100;  
  		
  //set prescalar to divide by 1024
  TCCR0B = 5;
  
  // turn on clear-on-match
  TCCR0A = (1 << WGM01) ;
  sei();
}

void loop()
{

  if (!queue.isEmpty ())
  {
    myData = queue.dequeue();
    if (myData.interrupt == 1)
    {
      Serial.println(myData.id);
      // Serial.print("  ");
      // Serial.print("TAP ");
      // Serial.println(myData.time);

    }
    else if (myData.interrupt == 2)
    {
      Serial.println(myData.id);
      //  Serial.print("  ");
      //  Serial.print("FF ");
      //  Serial.println(myData.time);

    }

  }

  // Read pattern from MATLAB
  pattern = Serial.read();
  if (pattern == 'a')
  {
    TIMSK0 = (0 << OCIE0A);
    radio.stopListening();
    LEDData.LEDnbr = 1;
    radio.write(&LEDData, sizeof(LEDData) );
    radio.startListening();
    counter = 0;
    TIMSK0 = (1 << OCIE0A);
  }
  else if (pattern == 'b')
  {
    TIMSK0 = (0 << OCIE0A);
    radio.stopListening();
    LEDData.LEDnbr = 2;
    radio.write(&LEDData, sizeof(LEDData) );
    radio.startListening();
    counter = 0;
    TIMSK0 = (1 << OCIE0A);
  }
  else if (pattern == 'c')
  {
    TIMSK0 = (0 << OCIE0A);
    radio.stopListening();
    LEDData.LEDnbr = 3;
    radio.write(&LEDData, sizeof(LEDData) );
    radio.startListening();
    counter = 0;
    TIMSK0 = (1 << OCIE0A);
  }

}// end main loop


